#!/usr/bin/perl -wT
# $Id: H2vogt.pl,v 1.6 2012/11/08 10:00:35 sofc Exp $
$ENV{'BASH_ENV'} = '/bin:/usr/bin:/usr/local/bin';
$ENV{'ENV'} = '/bin:/usr/bin:/usr/local/bin';
use strict;
use warnings;
use RFC::Rig;
use RFC::Main;
use RFC::Header;

my $type = 'H2mailfile';

if (@ARGV < 1)
{
    my $string = "Not enough input parameters for H2vogt.pl\n";
    $string = $string.'Usage: H2vogt.pl  Rig_number [opt: current]'."\n";
    $string = $string."If the \'current\' option is given, the rig is set\nto open circuit voltage upon low H2flow\nOtherwise only a mail is sent.\n";
    die "$string";
    exit 1;
}

my $rig = RFC::Rig->new(shift);
$rig->check_run;

my $current = 0;
foreach (@ARGV)
{
    $current = 1 if /current/i;
}

sleep(30);

my $homedir = '/home/rig'.$rig->name.'/';
my $currenttest = $rig->session;
my $datadir = $homedir . $rig->name . 'test'. $currenttest;
my $lockfile = SemaforeFile->new('/tmp/'.$$.'.txt',$datadir.'/lockfile_H2vogt');
my $res = $lockfile->lock_sh_nb();
exit unless ($res);

my $time = localtime;
my $file = $RFC::Header::publicdir.'rig'.$rig->name.'/'.$rig->name.'test'.$currenttest;
my $str = $rig->get_cv('main','vogt_gasses');
my @gaslist = ('h2','ch4');
@gaslist = split(/\s*\,\s*/,$str) if ($str);
$str = $rig->get_cv('main','additional_vogt_gasses_cutoff');
$str = $rig->get_cv('main','aditional_vogt_gasses_cutoff') unless ($str);
my @gaslistcut = @gaslist;
if ($str)
{
    foreach (split(/\s*\,\s*/,$str))
    {
	push @gaslistcut,$_;
    }
}

my $ok = 0;
my $resetok = 0;
my $string = '';
for my $gas (@gaslist)
{
    my $device = $rig->get_device('gas',$gas);
    $rig->die("Could not get device id for gas '$gas'") unless ($device);
    my $cutoff = $device->cutoff_set();
    my @list = ();
    for (0..4)
    {
	my $f = $device->read;
	### Fix for flowcontroller error ###
	next if ($device->type eq 'Brooks' && $f == -1); 
	push @list,$f if (defined($f));
    }
    unless (@list)
    {
	$rig->warning_mail($type,"Could not determine gas flow for gas $gas on rig $rig at $time");
	exit;
    }
    @list = sort {$a <=> $b} @list;
    my $flow = $list[POSIX::floor(@list/2+0.6)];
    ### Fix fore real failure        ###
    if ($flow < $cutoff)
    {
	$string .= ucfirst($gas)." flow was below $cutoff L/h, measured flow: $flow L/h\n";
    }
    else
    {
	$ok = 1;
	$resetok = 1 if (defined($flow) && $flow > $cutoff * 1.5);
    }
}


unless ($ok || $rig->check_warning($type))
{
    if ($current)
    {
	$string .= "Current set to OCV\n";
	my $str = $rig->get_cv('main','iv_control_names');
	if ($str)
	{
	    foreach my $psu (split(//,$str))
	    {
		$rig->current('ocv',$psu,$0);
	    }
	}
	else
	{
	    $rig->current('ocv');
	}
	### Stop any running program! ####
	$string .= "Stopping any running program!\n";
	RFC::Main::CGI_client($rig->name,'stop_custom');
	sleep(5);
    }
    for my $gas (@gaslistcut)
    {
	my $device = $rig->gas($gas);
	$rig->die("Could not get device id for gas '$gas'") unless ($device);
	$device->setflow(0,$0);
	$string = $string . "$gas flow set to 0 automatically by $0\n";
    }
    if ($rig->get_cv('main','additional_gas_trip_program'))
    {
	my $cmd = $RFC::Header::libdir.'logprograms/'.$rig->get_cv('main','additional_gas_trip_program');
	if (-e $cmd && -x $cmd)
	{
	    $string .= 'Executing additional gas trip program: '.$cmd;
	    system($cmd.' '.$rig->name);
	}
    }
    $rig->warning_mail($type,$string);
}

#### Remove mailsend file if flow is back to normal ###
$rig->unset_warning($type) if ($resetok);


