#!/usr/bin/perl -wT
#$Id: ajax_interface.cgi,v 1.15 2015/01/24 11:37:30 sqko Exp $
use strict;
use warnings;
use RFC::Main;
use RFC::Rig;
use RFC::RFCCGI qw (get_CGI_value_clean);
use CGI qw(:standard :netscape);
use SocketClient;
use LWP::UserAgent;
use Unicode::String qw(latin1 utf8);
my $ac = get_CGI_value_clean('action');
my $f = get_CGI_value_clean('furnace');
my $run = get_CGI_value_clean('runnr');
my $rig = get_CGI_value_clean('rig');
my @list = ();
my $ssl = 0;
$ssl = 1 if (CGI::https());
my $listserver = get_cv('servers','listserver');
if ($listserver)
{
    chomp $listserver;
    my $str = &RFC::Main::report_client($listserver,'servers');
    my @l = split(/\t/,$str) if ($str && $str !~ /ERROR/);
    foreach (@l)
    {
	if (/([\w\.\-]+)/)
	{
	    push @list,$1;
	}
    }
}

unless ($ac)
{
    print header(-access_control_allow_origin => '*',-content_type=>'text/plain',-charset=>'utf-8');
    print <<END;

Oops.. no parameters, displaying interface definitions!

Common parameters:

Parameter 'action', determines which function to call (each described below)
Generally, the script returns OK on the first line if input is valid, and an errorstring if not.
Thus remember to check for the 'OK' string as part of the ajax response function.

If password protection is implemented the following additional parameters may be nescesarry for some of the functions.:
'name' Username, text
'pass' Password hash, text
'time' Time of last login, text
'jscheck', integer (must be 1)


Function description based on 'action' parameter:

Action 'show_details':
Returns a http redirect / refresh header pointing to the resulting server and directory
Parameters:
'furnace'  Rig number, integer 
'testnr'   Test number to display, integer


Action 'get_test_location':
Unlike 'show_details', this function call tests that the test specified actually is a 
valid test number on the server where the rig was found as it is not safe to assume that all test numbers less
than the current test are valid.
Returns a http redirect / refresh header pointing to the resulting server and directory
Parameters:
'furnace'  Rig number, integer 
'testnr'   Test number to display, integer

Action 'furnaces':
No parameters
Displays a list of possible rigs (all in cluster).


Action 'is_furnace':
Parameters:
'furnace'  Rig number to check if it exists, integer


Action 'active_furnace':
Parameters:
'furnace'  Rig number to check if it is valid for new data, integer


Action 'run_list':
Parameters:
'furnace'  Rig number, integer
Returns a list of valid test numbers for the specified rig on any of the servers on the cluster.

Action 'is_run':
Parameters:
'furnace'  Rig number, integer
'runnr'    Test number to check, integer
Checks if the runnumber specified is valid for the rig in question on any of the servers on the cluster.


Action 'imp_list':
Parameters:
'furnace'  Rig number, integer
'runnr'    Test number to check, integer
Returns a list of impedance files for the test in quiestion from the correct servers on the cluster.


Action 'imp_file':
Parameters:
'furnace'  Rig number, integer
'runnr'    Test number to check, integer
'filenr'   Fime number to get, use the line number from the list returned by imp_list (linenr is first integer on the line). 
Retruns the content of the impedance vfile specified from the corresponding server on the cluster.


Action 'run':
Parameters:
'furnace'  Rig number, integer
'runnr'    Test number, integer
Returns the temperature, ramprate and dwelltime for the specified run
As most RFCcontrol tests are non-trivial, the function returns 0,0,0 and a warning for manual check.


Action 'rig_session':
Parameters:
'rig'  Rig number, integer
'runnr'    Test number, integer
Checks if the specified rig and test number exists on THIS server.


Action 'rig_session_list':
Parameters:
'rig'  Rig number, integer
Returns a list of test numbers for the specified rig on THIS server (only returns valid tests for which the test directory exists)


Action 'rig_imp_list':
Parameters:
'rig'  Rig number, integer
'runnr'    Test number, integer
Returns a list of impedance files from a valid rig and test on THIS server.


Action 'rig_imp_file':
'rig'  Rig number, integer
'runnr'    Test number to check, integer
'filenr'   Fime number to get, use the line number from the list returned by imp_list (linenr is first integer on the line). 
Retruns the content of the impedance file specified (must exist on THIS server).





END
    exit;
}

print header(-access_control_allow_origin => '*',-charset=>'utf-8');
if ($ac eq 'get_test_location')
{
    my $server = '';
    my $test = get_CGI_value_clean('testnr');
    if (!$f || !$test)
    {
	print "Warning\nNo rig or test number specified!\n";
	exit;	
    }
    foreach my $s (@list)
    {
	my $str = &RFC::Main::report_client($s,'is_test',$f,$test);
	next unless ($str);
	$server = $s;
	last;
    }
    if (!$server || !$f)
    {
	print "Warning\nNo rig found!\n";
	exit;
    }
    my $url =  $server.'/rig'.$f.'/'.$f.'test'.$test.'/';
    my $string = '<META HTTP-EQUIV="Refresh" CONTENT="0; URL=http://'.$url.'">';
    $string = '<META HTTP-EQUIV="Refresh" CONTENT="0; URL=https://'.$url.'">' if ($ssl);
    print $string,"\n";
    exit;
}


if ($ac eq 'show_details')
{
    my $server = '';
  OUTER:
    foreach my $s (@list)
    {
	my $str = &RFC::Main::report_client($s,'active_rigs');
	if (defined($str))
	{
	    chomp $str;
	    chomp $str;
	    chomp $str;
	    my @l = split(/\t/,$str);
	    foreach my $r (@l)
	    {
		next unless ($f == $r);
		$server = $s;
		last OUTER;
	    }
	}
    }
    if (!$server)
    {
	print "Warning\nNo rig found!\n";
	exit;
    }
    my $test = get_CGI_value_clean('testnr');
    my $url =  $server.'/rig'.$f.'/'.$f.'test'.$test.'/';
    my $string = '<META HTTP-EQUIV="Refresh" CONTENT="0; URL=http://'.$url.'">';
    $string = '<META HTTP-EQUIV="Refresh" CONTENT="0; URL=https://'.$url.'">' if ($ssl);
    print $string,"\n";
    exit;
}


if ($ac eq 'furnaces')
{
    print "OK\n";
    my @rigs = ();
    foreach my $s (@list)
    {
	my $str = &RFC::Main::report_client($s,'rigs');
	if (defined($str))
	{
	    chomp $str;
	    chomp $str;
	    chomp $str;
	    push @rigs,split(/\t/,$str);
	}
    }
    print join("\n",(sort {$a <=> $b} @rigs)),"\n";
    exit;
}

if ($ac eq 'is_furnace')
{
    foreach my $s (@list)
    {
	my $str = &RFC::Main::report_client($s,'rigs');
	if (defined($str))
	{
	    chomp $str;
	    chomp $str;
	    chomp $str;
	    my @l = split(/\t/,$str);
	    foreach (@l)
	    {
		next unless ($_ == $f);
		print "OK\n";
		print "1\n";
		exit;
	    }
	}    
    }
    print "Warning\n";
    print "No furnace found\n";
    exit;
}

if ($ac eq 'active_furnace')
{
    foreach my $s (@list)
    {
	my $str = &RFC::Main::report_client($s,'active_rigs');
	if (defined($str))
	{
	    chomp $str;
	    chomp $str;
	    chomp $str;
	    my @l = split(/\t/,$str);
	    foreach (@l)
	    {
		next unless ($_ == $f);
		print "OK\n";
		print "1\n";
		exit;
	    }
	}    
    }
    print "Warning\n";
    print "No furnace found\n";
    exit;
}

if ($ac eq 'run_list')
{
    my $res = "Warning\nNo rig found!\n";
  OUTER:
    foreach my $s (@list)
    {
	my $str = &RFC::Main::report_client($s,'rigs');
	if (defined($str))
	{
	    chomp $str;
	    chomp $str;
	    chomp $str;
	    my @l = split(/\t/,$str);
	    foreach my $r (@l)
	    {
		next unless ($f == $r);
		my $ua = LWP::UserAgent->new();
		$ua->timeout(60);
		my $url = 'http://'.$s . '/cgi-bin/celltest/ajax_interface.cgi?action=rig_session_list&rig=' . $r;
		$url = 'https://'.$s . '/cgi-bin/celltest/ajax_interface.cgi?action=rig_session_list&rig=' . $r if ($ssl);
		my $request = new HTTP::Request('GET',$url);
		my $response = $ua->request($request);
		my $content = $response->content();
		$res = $content if ($content);
		last OUTER;
	    }
	}
    }
    print $res;
}

if ($ac eq 'is_run')
{
    my $res = "Warning\nNo rig found!\n";
  OUTER:
    foreach my $s (@list)
    {
	my $str = &RFC::Main::report_client($s,'rigs');
	if (defined($str))
	{
	    chomp $str;
	    chomp $str;
	    chomp $str;
	    my @l = split(/\t/,$str);
	    foreach my $r (@l)
	    {
		next unless ($f == $r);
		my $ua = LWP::UserAgent->new();
		$ua->timeout(60);
		my $url = 'http://'.$s . '/cgi-bin/celltest/ajax_interface.cgi?action=rig_session&rig=' . $r . '&runnr=' . $run;
		$url = 'https://'.$s . '/cgi-bin/celltest/ajax_interface.cgi?action=rig_session&rig=' . $r . '&runnr=' . $run if ($ssl);
		my $request = new HTTP::Request('GET',$url);
		my $response = $ua->request($request);
		my $content = $response->content();
		$res = $content if ($content);
		last OUTER;
	    }
	}
    }
    print $res;
}

if ($ac eq 'imp_list')
{
    my $res = "Warning\nNo rig found!\n";
  OUTER:
    foreach my $s (@list)
    {
	my $str = &RFC::Main::report_client($s,'rigs');
	if (defined($str))
	{
	    chomp $str;
	    chomp $str;
	    chomp $str;
	    my @l = split(/\t/,$str);
	    foreach my $r (@l)
	    {
		next unless ($f == $r);
		my $ua = LWP::UserAgent->new();
		$ua->timeout(60);
		my $url = 'http://'.$s . '/cgi-bin/celltest/ajax_interface.cgi?action=rig_imp_list&rig=' . $r . '&runnr=' . $run;
		$url = 'https://'.$s . '/cgi-bin/celltest/ajax_interface.cgi?action=rig_imp_list&rig=' . $r . '&runnr=' . $run if ($ssl);
		my $request = new HTTP::Request('GET',$url);
		my $response = $ua->request($request);
		my $content = $response->content();
		$res = $content if ($content);
		last OUTER;
	    }
	}
    }
    print $res;
}



if ($ac eq 'imp_file')
{
    my $filenr = get_CGI_value_clean('filenr');
    my $res = "Warning\nNo rig found!\n";
  OUTER:
    foreach my $s (@list)
    {
	my $str = &RFC::Main::report_client($s,'rigs');
	if (defined($str))
	{
	    chomp $str;
	    chomp $str;
	    chomp $str;
	    my @l = split(/\t/,$str);
	    foreach my $r (@l)
	    {
		next unless ($f == $r);
		my $ua = LWP::UserAgent->new();
		$ua->timeout(60);
		my $url = 'http://'.$s . '/cgi-bin/celltest/ajax_interface.cgi?action=rig_imp_file&rig=' . $r . '&runnr=' . $run . '&filenr=' . $filenr;
		$url = 'https://'.$s . '/cgi-bin/celltest/ajax_interface.cgi?action=rig_imp_file&rig=' . $r . '&runnr=' . $run . '&filenr=' . $filenr if ($ssl);
		my $request = new HTTP::Request('GET',$url);
		my $response = $ua->request($request);
		my $content = $response->content();
		$res = $content if ($content);
		last OUTER;
	    }
	}
    }
    print $res;
}


if ($ac eq 'run')
{
    print "OK\n";
    print "0\n";
    print "0\n";
    print "0\n";
    print "Warning: Not a simple furnace run, check manually on server!\n";
}

if ($ac eq 'rig_session')
{
    my $r = RFC::Rig->new($rig);
    my $session = $r->session();
    if ($run > 0 && $run <= $session)
    {
	print "OK\n";
	print "1\n";
	exit;	
    }
    else
    {
	print "Warning\n";
	print "Run $run for rig $rig not valid!\n";
    }
}

if ($ac eq 'rig_session_list')
{
    my $r = RFC::Rig->new($rig);
    my $session = $r->session();
    print "OK\n";
    for (my $x = 1; $x < $session; $x++)
    {
	my $dir = $r->webdir($x);
	print $x,"\n" if (-d $dir);
    }
}

if ($ac eq 'rig_imp_list')
{
    my $r = RFC::Rig->new($rig);
    my $session = $r->session();
    if ($run > 0 && $run <= $session)
    {
	print "OK\n";
	my $filename = $r->webdir($run).'/impedance/imp_data';
	my $file = SemaforeFile->new($filename);
	my @list = $file->readlines();
	foreach (my $x=0; $x <  @list; $x++) 
	{
	    $list[$x] =~ s/^[\s\t]+//;
	    print $x.' '.$list[$x],"\n";
	}
	exit;	
    }
    else
    {
	print "Warning\n";
	print "Run $run for rig $rig not valid!\n";
    }
}

if ($ac eq 'rig_imp_file')
{
    my $r = RFC::Rig->new($rig);
    my $session = $r->session();
    my $filenr = get_CGI_value_clean('filenr');
    if ($run > 0 && $run <= $session)
    {
	my $filename = $r->webdir($run).'/impedance/imp_data';
	my $file = SemaforeFile->new($filename);
	my @list = $file->readlines();
	if ($list[$filenr] && $list[$filenr] =~ /([\w\.\-\:\,]+i2b)+\s+/)
	{
	    my $ifile = SemaforeFile->new($r->webdir($run).'/impedance/'.$1);
	    if ($ifile->exist)
	    {
		print "OK\n";
		print join("\n",$ifile->readlines);
	    }
	    else
	    {
		print "Warning\n";
		print "Impedance file $filenr not found for rig $rig and test $run!\n";	    
	    }
	}
	else
	{
	    print "Warning\n";
	    print "Impedance file $filenr not found for rig $rig and test $run!\n";	    
	}
	exit;	
    }
    else
    {
	print "Warning\n";
	print "Run $run for rig $rig not valid!\n";
    }
}
